import os
import time
import shutil
import unittest

from adamspy.adamspy import set_n_threads, get_n_threads, solve, XMT_PATTERN, modify_xmt_path

from test import *

TEST_ADM_FILE_TO_CHANGE = os.path.join(os.path.split(TEST_ADM_FILE)[0], 'test_modified.adm')

class Test_SetNThreads(unittest.TestCase):

    def setUp(self):
        # Delete the modified adm file if it exists
        try:
            os.remove(TEST_ADM_FILE_TO_CHANGE)
        except FileNotFoundError:
            pass
            
        # Copy the adm file
        shutil.copyfile(TEST_ADM_FILE, TEST_ADM_FILE_TO_CHANGE)               

    def test_set_n_threads(self):
        """Tests that set_n_threads correctly changes the adm file.

        """
        set_n_threads(TEST_ADM_FILE_TO_CHANGE, 10)
        with open(TEST_ADM_FILE_TO_CHANGE, 'r') as fid:
            lines = fid.readlines()
        
        threads_line = lines[12336].strip()

        self.assertEqual(threads_line, ', NTHREADS = 10')           

    def test_get_n_threads(self):
        """Tests that get_n_threads correctly gets the number of threads defined in an adm file.

        """
        n_threads = get_n_threads(TEST_ADM_FILE)
        self.assertEqual(n_threads, 4)

    def tearDown(self):
        # Delete the modified adm file if it exists
        try:
            os.remove(TEST_ADM_FILE_TO_CHANGE)
        except FileNotFoundError:
            pass
            
class Test_Solve(unittest.TestCase):

    def setUp(self):
        for file in glob.glob(os.path.join(EXISTING_MODEL_DIR, '*')):
            ext = os.path.splitext(file)[-1]
            if ext not in ['.acf', '.adm', '.xmt_txt']:
                try:
                    os.remove(file)
                except FileNotFoundError:               
                    pass
                except PermissionError:
                    pass

    def test_solve_wait(self):
        """Tests that adamspy.adamspy.solve() works when wait=True
        
        """
        acf_file = glob.glob(os.path.join(EXISTING_MODEL_DIR, '*.acf'))[0]
        solve(acf_file, wait=True)
        res_files = glob.glob(os.path.join(EXISTING_MODEL_DIR, '*.res'))
        self.assertTrue(res_files != [])

    
    def test_solve_dont_wait(self):
        """Tests that adamspy.adamspy.solve() works when wait=False
        
        """
        acf_file = glob.glob(os.path.join(EXISTING_MODEL_DIR, '*.acf'))[0]
        proc = solve(acf_file, wait=False)
        
        # Check that the res file isn't there yet
        res_files = glob.glob(os.path.join(EXISTING_MODEL_DIR, '*.res'))
        if res_files != []:
            self.fail('Solve did not wait!')
        
        # Wait 60 seconds for the res file to show up
        for _i in range(60):            
            res_files = glob.glob(os.path.join(EXISTING_MODEL_DIR, '*.res'))
            if res_files != []:
                break 
            time.sleep(1)
    
        self.assertTrue(res_files != [])
       
    def tearDown(self):
        for file in glob.glob(os.path.join(EXISTING_MODEL_DIR, '*')):
            ext = os.path.splitext(file)[-1]
            if ext not in ['.acf', '.adm', '.xmt_txt']:
                try:
                    os.remove(file)
                except FileNotFoundError:               
                    pass
                except PermissionError:
                    pass

class Test_CMD(unittest.TestCase):

    def setUp(self):
        return

    def test_xmt_pattern(self):
        xmt_line = '   file_name = "FRS3EQ_case1_LAUL.xmt_txt"  &'
        match = XMT_PATTERN.match(xmt_line)

        self.assertTrue(bool(match))
    
    def test_modify_xmt_path(self):
        path_to_change_to = os.path.join('path', 'to', 'bogus.xmt_txt')
        original_path = modify_xmt_path(TEST_CMD_XMT, path_to_change_to)
        new_path = modify_xmt_path(TEST_CMD_XMT, original_path)

        self.assertListEqual([original_path, new_path], ['FRS3EQ_case1_LAUL.xmt_txt', path_to_change_to])

    def tearDown(self):
        return
        


import os
import unittest

from adamspy.postprocess.animation import create_animation
from adamspy.postprocess import _get_model_name_from_cmd

from test import TEST_EXISTING_GENERIC_CMD_MODEL, TEST_EXISTING_GENERIC_RES_FILE, TEST_EXISTING_GENERIC_RES_FILE_ANALYSIS_NAME

class Test_Animation(unittest.TestCase):

    def setUp(self):        
        self.anim_file = create_animation(TEST_EXISTING_GENERIC_CMD_MODEL, TEST_EXISTING_GENERIC_RES_FILE, _get_model_name_from_cmd(TEST_EXISTING_GENERIC_CMD_MODEL), TEST_EXISTING_GENERIC_RES_FILE_ANALYSIS_NAME)

    def test_animation_file_created(self):
        self.assertTrue(os.path.isfile(self.anim_file))

    def tearDown(self):
        return



import os

import numpy as np
import Adams                            # pylint: disable=import-error

def edit_results(file, reqs_to_edit, output_file):                             #pylint: disable=redefined-outer-name
    """Opens Adams/View, loads results files, and writes the specified results to an output file.
    
    Parameters
    ----------
    file : list
        List of results or requests filenames
    reqs_to_edit : dict
        Dictionary of results to edit.  Should include time in the root dictionary.

    """        
    # Load Results    
    if os.path.exists(os.path.splitext(file)[0] + '.adm'):
        Adams.execute_cmd('file adams_data_set  read file_name="{}"'.format(os.path.splitext(file)[0] + '.adm'))
        Adams.execute_cmd('file analysis read file_name="{}" model_name={}'.format(file, os.path.splitext(os.path.split(file)[-1])[0]))
    else:
        Adams.execute_cmd('file analysis read file_name="{}"'.format(file))

    _mod, ans = get_mod_and_ans()

    if 'TIME' in reqs_to_edit:
        time_key = 'TIME'
    elif 'time' in reqs_to_edit:
        time_key = 'time'
    else:
        time_key = None
        
    print('----- START: DEBUG MESSAGE ----')
    print(f'time_key = {time_key}')
    print('----- STOP : DEBUG MESSAGE ----')

    if time_key is not None:
        time = ans.results.get('TIME').values
        _time_np = np.round(np.asarray(time), 10)
        i_min = np.argmax(_time_np >= float(np.round(reqs_to_edit[time_key][0], 10)))
        
        print('----- START: DEBUG MESSAGE ----')
        print(f'reqs_to_edit[time_key][-1] = {reqs_to_edit[time_key][-1]}')
        print('----- STOP : DEBUG MESSAGE ----')
    
        i_max = np.argmax(_time_np > float(np.round(reqs_to_edit[time_key][-1], 10)))

        print('----- START: DEBUG MESSAGE ----')
        print(f'i_min = {i_min}')
        print(f't_min = {time[i_min]}')
        print(f'i_max = {i_max}')
        print(f't_max = {time[i_max]}')
        print('----- STOP : DEBUG MESSAGE ----')

    for (req_to_edit, cmps_to_edit) in [(req, cmps) for req, cmps in reqs_to_edit.items() if req != time_key]:
        
        if req_to_edit not in ans.results:
            raise AviewError(f'{req_to_edit} is not a result set!')

        for cmp_to_edit, values in cmps_to_edit.items():
            if cmp_to_edit not in ans.results[req_to_edit]:
                raise AviewError(f'{cmp_to_edit} is not a component in {req_to_edit}!')
                        
            new_values = values if time_key is None else ans.results[req_to_edit][cmp_to_edit].values[:i_min] + values + ans.results[req_to_edit][cmp_to_edit].values[i_max:]
            edit_result_in_DB(req_to_edit, cmp_to_edit, new_values)

    write_edited_results(output_file)

def write_edited_results(filename):
    _mod, ans = get_mod_and_ans()
    Adams.execute_cmd(f'file results write analysis_name="{ans.name}" file_name="{filename}"')
           
def edit_result_in_DB(res : str, cmp : str, new_values):  
    _mod, ans = get_mod_and_ans()
    orig_len = len(ans.results[res][cmp].values)
    
    # Confirm that lengths are the same
    if orig_len != len(new_values):
        raise AviewError(f'Original {res}.{cmp} had {orig_len} values and you are trying to replace them with {len(new_values)} values!')

    Adams.execute_cmd(f'numeric_result component modify result_set_component_name = {res}.{cmp} values = ' + f'{new_values}'.replace('[', '',).replace(']', ''))

def get_mod_and_ans():
    # Get the model object
    mod = Adams.Models.get([mn for mn in Adams.Models][0])
    
    # Get the analysis object
    ans = mod.Analyses.get([an for an in mod.Analyses][0])
    return mod, ans

def get_result_file_type(file):        
    """Checks if `file` is a results file or a request file.
    
    Parameters
    ----------
    file : str
        Results/request file name.
    
    Returns
    -------
    str
        Type of file
    
    Raises
    ------
    ValueError
        Raised if `file` is neither a request nor a results file.

    """
    if file.endswith('.res'):
        file_type = 'results'
        print('\n--------------\nResults file detected!\n--------------')
    elif file.endswith('.req'):
        file_type = 'request'
        print('\n--------------\nRequst file detected!\n--------------')
    else:
        raise AviewError('Files must be .res or .req files.')
    return file_type

def get_req_from_comment(comment, ans):
    """Gets the actual name of a request
    
    Parameters
    ----------
    comment : str
        Request comment
    results : Analysis
        Adams `:obj:Analysis` object
    
    Returns
    -------
    nt
        Adams results (`:obj:nt`)? object
        
    """
    for req_name in [r for r in ans.results.keys() if r.startswith('REQUEST_')]:
        if Adams.evaluate_exp(f'{req_name}.comment').lower() == comment.lower():
            return ans.results.get(req_name)

    raise AviewError(f'No request found with comment {comment}!')

class AviewError(Exception):
    """Use this error so ppt._get_log_errors will flag these errors.    
    
    """
    pass
            
if __name__ in ['aview_main', '__main__']:
    
    Adams.execute_cmd('def com echo=on')

    res_file = r'{{ res_file }}'
    reqs_to_edit = {{ reqs_to_edit }}
    output_file = r'{{ output_file }}'

    edit_results(res_file, reqs_to_edit, output_file)
    


import re
from io import StringIO

from numpy import ceil, sqrt
import pandas as pd


MODE_LABEL = re.compile('^\\s*Energy table for MODE\\s*\\d+$', flags=re.MULTILINE)
MODAL_SCALAR_LABELS = {
    'zeta': re.compile('^\\s*Damping ratio\\s*=.*$', flags=re.MULTILINE),
    'w_n': re.compile('^\\s*Undamped natural freq\\.\\s*=.*$', flags=re.MULTILINE),
    'K': re.compile('^\\s*Generalized stiffness\\s*=.*$', flags=re.MULTILINE),
    'E': re.compile('^\\s*Kinetic energy\\s*=.*$', flags=re.MULTILINE)
}

MODAL_TABLE_LABELS = {
    'coords': re.compile('^\\s*Normalized coordinates in this mode\\s*$', flags=re.MULTILINE),
    'KE': re.compile('^\\s*Percentage distribution of Kinetic energy\\s*$', flags=re.MULTILINE),
    'SE': re.compile('^\\s*Percentage distribution of strain energy\\s*$', flags=re.MULTILINE),
    'DE': re.compile('^\\s*Percentage distribution of dissipative energy\\s*$', flags=re.MULTILINE)    
}

# MODE_TABLE_HORIZ_SEP = re.compile('^\\s*\\+-+\\+\\s*$', flags=re.MULTILINE)
MODE_TABLE_HORIZ_SEP = re.compile('^\\s*[\\+-]+\\s*$', flags=re.MULTILINE)

class Modal():

    def __init__(self, modes: dict):
        self.modes = modes

    def get_modes_where_parts_participate(self, part_suffix, table, column, threshold=.9, sum_or_max='max'):
        """Returns a dictionary where the keys are mode numbers and the values are `:obj:Mode` objects in which any part with the suffix `part_suffix` has at least `threshold` percent of the total.

        Parameters
        ----------
        part_suffix : str
            Suffix of parts to check for
        table : str
            Table type. Options are 'coords', 'kinetic_energy', 'strain_energy', 'dissipative_energy'
        column : str
            Column name from the table
        threshold : float
            Part's value must be greater than `threshold` multiplied the sum or max (depending on `sum_or_max` value) of the column in the table to be considered participating
        sum_or_max : str
            Must be 'sum' or 'max'

        Returns
        -------
        dict
            Dictionary where the keys are mode numbers and the values are `:obj:Mode` objects

        """ 
        modes_with_participation = {}

        for mode_number, mode in self.modes.items():
            
            # Sort the kinetic energy table by decending sum
            tbl_obj = getattr(mode, table)
            sorted_tbl = tbl_obj.sort_values(column, ascending=False).reset_index(drop=True)              
            
            # Get the value to compare against
            value = sorted_tbl[column].max() if sum_or_max.lower() == 'max' else sorted_tbl[column].sum()
            
            filtered_tbl = sorted_tbl[sorted_tbl['view_name'].str.split('.', expand=True)[1].str.startswith(part_suffix)].reset_index(drop=True)

            if filtered_tbl[column][0] >= value*threshold:
                modes_with_participation[mode_number] = mode
            
        return modes_with_participation

class Mode():
    def __init__(self, scalars: dict, tables: dict):
        self.zeta = scalars['zeta']
        self.w_n = scalars['w_n']
        self.K = scalars['K']
        self.E = scalars['E']
        self.coords = tables['coords'] if 'coords' in tables else None
        self.kinetic_energy = tables['KE'] if 'KE' in tables else None
        self.strain_energy = tables['SE'] if 'SE' in tables else None
        self.dissipative_energy = tables['DE'] if 'DE' in tables else None

        if self.coords is not None:
            self._add_mag_to_coords()

    def _add_mag_to_coords(self):
        self.coords['M_magnitude'] = sqrt(self.coords['X_magnitude']**2 + self.coords['Y_magnitude']**2 + self.coords['Z_magnitude']**2)
        self.coords['RM_magnitude'] = sqrt(self.coords['RX_magnitude']**2 + self.coords['RY_magnitude']**2 + self.coords['RZ_magnitude']**2)

def parse_table_file(filename):
    with open(filename, 'r') as fid:
        lines = fid.readlines()
    
    mode_lines_dict = _split_by_mode(lines)

    modes_dict = {}
    for mode_number, mode_lines in mode_lines_dict.items():
        scalars_dict, tables_dict = _parse_mode_lines(mode_lines)
        modes_dict[mode_number] = Mode(scalars_dict, tables_dict)
    
    return Modal(modes_dict)
    
def _split_by_mode(lines: list) -> dict:    
    
    modes = []
    start_lines = []

    # Get the mode numbers and the start lines
    for i_line, line in enumerate(lines):
        if MODE_LABEL.match(line):
            mode_number = int(line.strip().replace('\n','').split(' ')[-1])
            start_line = i_line - 1

            modes.append(mode_number)
            start_lines.append(start_line)

    # Create a dictionary of lists of lines for each mode
    mode_lines = {}
    for mode, start_line, stop_line in zip(modes, start_lines, [sl for sl in start_lines[1:]]+[-1]):
        mode_lines[mode] = lines[start_line:stop_line]
    
    return mode_lines

def _parse_mode_lines(mode_lines: list) -> (dict, dict):
    """Given a list of strings representing lines in an Adams/Linear modal output file for a single mode, returns dictionaries of the modal information.

    Parameters
    ----------
    mode_lines : list
        List of strings representing lines in an Adams/Linear modal output file for a single mode.

    Returns
    -------
    dict
        Dictinary of the scalars that describe the mode
    dict
        Dictionary of dataframes of the the lines for each mode table. 

    """    
    mode_scalars_dict = {}
    table_names = []
    tbl_st_ln_nums = []

    for i_line, line in enumerate(mode_lines):
        for scalar_name, pattern in MODAL_SCALAR_LABELS.items():
            if pattern.match(line):
                mode_scalars_dict[scalar_name] = float(line.split('=')[-1].strip().split(' ')[0])
                break
        
        for table_name, pattern in MODAL_TABLE_LABELS.items():
            if pattern.match(line):
                table_names.append(table_name)
                tbl_st_ln_nums.append(i_line)
                break
            
    # Create a dictionary of pandas dataframes
    tbl_dict = {}
    for tbl_name, st_ln_num, sp_ln_num in zip(table_names, tbl_st_ln_nums, [sl for sl in tbl_st_ln_nums[1:]]+[None]):

        # Get a pandas dataframe of the table
        tbl_dict[tbl_name] = _parse_table_lines(mode_lines[st_ln_num:sp_ln_num]) if sp_ln_num is not None else _parse_table_lines(mode_lines[st_ln_num:])

    return mode_scalars_dict, tbl_dict

def _parse_table_lines(table_lines: list):
    """Given a list of strings representing lines in an Adams/Linear modal output file for a single mode, returns a pandas table of the data


    Parameters
    ----------
    table_lines : list
        list of strings representing lines in an Adams/Linear modal output file for a single mode

    Returns
    -------
    pd.DataFrame
        Pandas `:obj:pd.DataFrame` representing the mode table

    """    
    idx_horiz_sep = [i for i,line in enumerate(table_lines) if MODE_TABLE_HORIZ_SEP.match(line)][:3]

    # Get the header lines should be one or two lines
    header_lines = table_lines[idx_horiz_sep[0]+1:idx_horiz_sep[1]]

    # Get the column names
    column_names = _get_column_names(header_lines)

    # Prep lines for input to read csv by removing | and replaceing white space with commas
    prepped_lines = [re.sub('\\s{1,11}', ',', line.replace('|', '').strip()) for line in table_lines[idx_horiz_sep[1]+1:idx_horiz_sep[2]]]
    table_text = '\n'.join(prepped_lines)

    # Get the table as a dataframe
    table = pd.read_csv(StringIO(table_text), names=column_names)

    return table

def _get_column_names(header_lines):    
    if len(header_lines) == 2:
        # If the header has two lines
        first_headers_list = re.split('\\s+', header_lines[0].replace('|', '').strip())
        second_headers_list = re.split('\\s+', header_lines[1].replace('|', '').strip())

        column_names = ['solver_name'] + ['{}_{}'.format(first_headers_list[int(ceil((i+1)/2)-1)], sec_head) for i, sec_head in enumerate(second_headers_list)] + ['view_name']

    elif len(header_lines) == 1:
        # If the header has one line
        column_names = ['solver_name'] + re.split('\\s+', header_lines[0].replace('|', '').strip()) + ['view_name']
    
    return column_names


import os
import unittest

from adamspy.postprocess.exported import import_spreadsheet

FILENAME = os.path.join(os.getcwd(), 'test', 'files', 'exported', 'lc4_drv_tq.csv')

class Test_Exported_DataFrame(unittest.TestCase):

    def setUp(self):
        self.data = import_spreadsheet(FILENAME, output_type='dataframe')

    def test_column_names(self):
        self.assertListEqual(list(self.data.columns), ['FX', 'FY', 'FZ', 'TX', 'TY', 'TZ', 'FMAG', 'TMAG'])
    
    def test_index_name(self):
        self.assertEqual(self.data.index.name, 'TIME')    

    def tearDown(self):
        return

class Test_Exported_Dict(unittest.TestCase):

    def setUp(self):
        self.data = import_spreadsheet(FILENAME, output_type='dict')

    def test_keys(self):
        self.assertListEqual([nm for nm in self.data.keys()], ['TIME', 'FX', 'FY', 'FZ', 'TX', 'TY', 'TZ', 'FMAG', 'TMAG'])

    def tearDown(self):
        return


"""Package for working with Adams Results

"""
import os
import subprocess
import re
import time

PPT_AFTERSTART_FILENAME = 'pptAS.cmd'
AVIEW_AFTERSTART_FILENAME = 'aviewAS.cmd'
RES_LOADED_PATTERN = '! File Name:.*{}.*Time Steps:.*Start Time:.*Stop Time:.*(sec)'
CMD_MODNAME_PATTERN = r'model create[ \t]+&[ \t]*\n[ \t]*model_name[ \t]*=[ \t]*\w+[ \t]*'

def launch_ppt(res_file, cmd_file=None, wait=False, timeout=30, _terminate=False):
    """Launches the Adams PostProcessor and reads in the specified results file.
    
    Parameters
    ----------
    res_file : str
        Filepath to an Adams Results file.
    cmd_file : str, optional
        Adams View command (.cmd) file. If given, this will be loaded before the results file.  This is necessary to view animations.
    wait : bool, optional
        If `True`, code execution will freeze until the postprocessor is closed. (the default is False)
    timeout : float, optional
        If `wait`=`False`, code execution will freeze until the postprocessor has started loading the results file or until `timeout` secodns have elapsed.
    _terminate : bool, optional
        If `True`, the PostProcessor will close immediately.  This is for testing purposes.  
    
    """    
    # Determine the directory, relative results filename, and pptAS name
    directory, res_file = os.path.split(res_file)
    if not directory:
        directory = os.getcwd()    
    ppt_as_filename = os.path.join(directory, AVIEW_AFTERSTART_FILENAME)

    
    # Write the pptAS file
    with open(ppt_as_filename, 'w') as fid:        
        if cmd_file is not None and os.path.isfile(os.path.join(directory, cmd_file)):
            # if the .cmd file is specified, get the model name from the cmd file
            model_name = _get_model_name_from_cmd(cmd_file)
            
            # Write the command to load the .cmd file
            fid.write(f'file command read file_name="{cmd_file}"\n')

            # Write the command to load the results file
            fid.write(f'file results read model_name={model_name} file_name="{res_file}"\n')   

            # Open the postprocess window
            fid.write(f'interface plot window open')
        
        else:
            # if the .cmd file is not specified, write the command to load the results file
            fid.write(f'file results read file_name="{res_file}"\n')    

            # Open the postprocess window
            fid.write(f'interface plot window open')
        
    # Set the ppt.log filename
    ppt_log_file = os.path.join(directory, 'aview.log')

    # Remove aview.log
    if os.path.exists(ppt_log_file):
        try:
            os.remove(ppt_log_file)
        except PermissionError:
            pass

    # Run the postprocessor
    startupinfo = subprocess.STARTUPINFO()
    startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW        
    ppt_proc = subprocess.Popen('"{}" aview ru-s i'.format(os.environ['ADAMS_LAUNCH_COMMAND']), cwd=directory, startupinfo=startupinfo)
    
    # Terminate immediately or wait for the process to complete before moving on.
    if _terminate:
        ppt_proc.terminate()
    elif wait:
        ppt_proc.wait()
    else:
        sleep_time = 0.2
        for _i in range(int(timeout/sleep_time)):
            
            ppt_log_file_exists = os.path.exists(ppt_log_file)       

            if ppt_log_file_exists is True:
                # If ppt.log exists, open it and see if the results have been loaded
                with open(ppt_log_file, 'r') as fid:
                    text = fid.read()
                if re.search(RES_LOADED_PATTERN.format(res_file), text):
                    break
                    
            time.sleep(sleep_time)

    # Remove the pptAS file
    try:
        os.remove(ppt_as_filename)
    except PermissionError:
        pass

    return directory

def _get_model_name_from_cmd(filename):
    """Gets the name of the model defined in the command file.
    
    Parameters
    ----------
    filename : str
        Filename of Adams View command (.cmd) file
    
    Returns
    -------
    str
        Name of model defined in the command file.

    """
    with open(filename, 'r') as fid:
        text = fid.read()

    mod_name_block = re.findall(CMD_MODNAME_PATTERN, text)[0]
    mod_name = mod_name_block.split('=')[-1].strip()

    return mod_name



import unittest
import os

import matplotlib.pyplot as plt

from adamspy.waterfall import waterfall

RESULTS_FILE = os.path.join(os.getcwd(), 'test', 'files', 'waterfall_files', 'LC_3.req')

RESULT = 'cust_response_2_vel'
RESULT_COMPONENT = 'X'
FIGURE_FOLDER = os.path.join(os.getcwd(), 'test', 'files', 'waterfall_files', 'figures')

class Test_Waterfall(unittest.TestCase):

    def setUp(self):
        self.picture_file = os.path.join(FIGURE_FOLDER, f'{os.path.splitext(os.path.split(RESULTS_FILE)[-1])[0]}_{RESULT}_{RESULT_COMPONENT}_.png')
        
        # Make waterfall
        self.fig = waterfall.fft_watefall(RESULTS_FILE, RESULT, RESULT_COMPONENT, percent_overlap=75, n_fft=128, t_min=6.25, t_max=10, input_res='response_1_vel', input_comp='R3', response_unit='mm/s', order_lines=[1, 4, 10, 40, 63], title=RESULT, z_scale='dB', f_range=[0, 500])
        
    def test_human_approval(self):
        plt.show()     
        self.assertTrue(True)
    
    def tearDown(self):
        plt.close()

        try:
            os.remove(self.picture_file)
        except FileNotFoundError:
            pass
